"""Type definitions for ADPA Core module."""

from typing import Dict, Any, List, Optional
from pydantic import BaseModel, Field


class CoreConfig(BaseModel):
    """Core configuration."""

    max_threads: int = Field(default=10, gt=0, description="Maximum number of worker threads")
    queue_size: int = Field(default=1000, gt=0, description="Maximum queue size")
    batch_size: int = Field(default=100, gt=0, description="Batch processing size")
    timeout: int = Field(default=30, gt=0, description="Operation timeout in seconds")

    class Config:
        """Pydantic model configuration."""

        frozen = True


class ProcessingResult(BaseModel):
    """Result of processing operation."""

    success: bool = Field(..., description="Whether processing was successful")
    data: Dict[str, Any] = Field(default_factory=dict, description="Processed data")
    error: Optional[str] = Field(None, description="Error message if processing failed")

    class Config:
        """Pydantic model configuration."""

        frozen = True


class WorkflowStep(BaseModel):
    """Workflow step definition."""

    name: str = Field(..., min_length=1, description="Step name")
    action: str = Field(..., min_length=1, description="Action to perform")
    dependencies: List[str] = Field(default_factory=list, description="Step dependencies")

    class Config:
        """Pydantic model configuration."""

        frozen = True


class Workflow(BaseModel):
    """Workflow definition."""

    steps: List[WorkflowStep] = Field(..., min_items=1, description="Workflow steps")

    class Config:
        """Pydantic model configuration."""

        frozen = True


class Event(BaseModel):
    """Event definition."""

    type: str = Field(..., min_length=1, description="Event type")
    data: Dict[str, Any] = Field(default_factory=dict, description="Event data")

    class Config:
        """Pydantic model configuration."""

        frozen = True
