"""
Database configuration management.
"""
from typing import Optional, Dict, Any
from pydantic import BaseModel, Field, validator
import os
from urllib.parse import quote_plus

class DatabaseConfig(BaseModel):
    """Database configuration settings."""
    
    host: str = Field(
        default="localhost",
        description="Database host"
    )
    port: int = Field(
        default=5432,
        description="Database port"
    )
    database: str = Field(
        description="Database name"
    )
    username: str = Field(
        description="Database username"
    )
    password: str = Field(
        description="Database password"
    )
    schema: str = Field(
        default="public",
        description="Database schema"
    )
    pool_size: int = Field(
        default=5,
        description="Connection pool size"
    )
    max_overflow: int = Field(
        default=10,
        description="Maximum number of connections that can be created beyond pool_size"
    )
    pool_timeout: int = Field(
        default=30,
        description="Number of seconds to wait before giving up on getting a connection from the pool"
    )
    pool_recycle: int = Field(
        default=1800,
        description="Number of seconds after which a connection is automatically recycled"
    )
    echo: bool = Field(
        default=False,
        description="If True, enables SQLAlchemy echo mode"
    )
    
    @validator("port")
    def validate_port(cls, v: int) -> int:
        """Validate port number."""
        if not 1 <= v <= 65535:
            raise ValueError("Port must be between 1 and 65535")
        return v
    
    @validator("pool_size")
    def validate_pool_size(cls, v: int) -> int:
        """Validate pool size."""
        if v < 1:
            raise ValueError("Pool size must be at least 1")
        return v
    
    @validator("max_overflow")
    def validate_max_overflow(cls, v: int) -> int:
        """Validate max overflow."""
        if v < 0:
            raise ValueError("Max overflow must be non-negative")
        return v
    
    @property
    def connection_string(self) -> str:
        """Get SQLAlchemy connection string.
        
        Returns:
            Database connection string
        """
        return (
            f"postgresql://{self.username}:{quote_plus(self.password)}"
            f"@{self.host}:{self.port}/{self.database}"
        )
    
    @property
    def engine_config(self) -> Dict[str, Any]:
        """Get SQLAlchemy engine configuration.
        
        Returns:
            Engine configuration dictionary
        """
        return {
            "pool_size": self.pool_size,
            "max_overflow": self.max_overflow,
            "pool_timeout": self.pool_timeout,
            "pool_recycle": self.pool_recycle,
            "echo": self.echo
        }
    
    @classmethod
    def from_env(cls) -> "DatabaseConfig":
        """Create configuration from environment variables.
        
        Returns:
            Database configuration
        
        Environment Variables:
            DB_HOST: Database host
            DB_PORT: Database port
            DB_NAME: Database name
            DB_USER: Database username
            DB_PASS: Database password
            DB_SCHEMA: Database schema
            DB_POOL_SIZE: Connection pool size
            DB_MAX_OVERFLOW: Maximum connection overflow
            DB_POOL_TIMEOUT: Pool timeout in seconds
            DB_POOL_RECYCLE: Pool recycle time in seconds
            DB_ECHO: SQLAlchemy echo mode
        """
        return cls(
            host=os.getenv("DB_HOST", "localhost"),
            port=int(os.getenv("DB_PORT", "5432")),
            database=os.getenv("DB_NAME", ""),
            username=os.getenv("DB_USER", ""),
            password=os.getenv("DB_PASS", ""),
            schema=os.getenv("DB_SCHEMA", "public"),
            pool_size=int(os.getenv("DB_POOL_SIZE", "5")),
            max_overflow=int(os.getenv("DB_MAX_OVERFLOW", "10")),
            pool_timeout=int(os.getenv("DB_POOL_TIMEOUT", "30")),
            pool_recycle=int(os.getenv("DB_POOL_RECYCLE", "1800")),
            echo=os.getenv("DB_ECHO", "").lower() == "true"
        )
