"""Initial database schema

Revision ID: 001
Create Date: 2025-02-09 13:10:08.000000
"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

# revision identifiers
revision = "001"
down_revision = None
branch_labels = None
depends_on = None

def upgrade():
    # Create users table
    op.create_table(
        "users",
        sa.Column("id", postgresql.UUID(as_uuid=True), primary_key=True),
        sa.Column("username", sa.String(50), unique=True, nullable=False),
        sa.Column("email", sa.String(255), unique=True, nullable=False),
        sa.Column("password_hash", sa.String(255), nullable=False),
        sa.Column("is_active", sa.Boolean(), nullable=False, default=True),
        sa.Column("last_login", sa.DateTime()),
        sa.Column("preferences", postgresql.JSON()),
        sa.Column("created_at", sa.DateTime(), nullable=False),
        sa.Column("updated_at", sa.DateTime(), nullable=False)
    )
    
    # Create queries table
    op.create_table(
        "queries",
        sa.Column("id", postgresql.UUID(as_uuid=True), primary_key=True),
        sa.Column("user_id", postgresql.UUID(as_uuid=True), nullable=False),
        sa.Column("natural_query", sa.Text(), nullable=False),
        sa.Column("sql_query", sa.Text(), nullable=False),
        sa.Column("execution_time", sa.Integer()),
        sa.Column("row_count", sa.Integer()),
        sa.Column("status", sa.String(20), nullable=False),
        sa.Column("error_message", sa.Text()),
        sa.Column("metadata", postgresql.JSON()),
        sa.Column("created_at", sa.DateTime(), nullable=False),
        sa.Column("updated_at", sa.DateTime(), nullable=False),
        sa.ForeignKeyConstraint(["user_id"], ["users.id"])
    )
    
    # Create query_results table
    op.create_table(
        "query_results",
        sa.Column("id", postgresql.UUID(as_uuid=True), primary_key=True),
        sa.Column("query_id", postgresql.UUID(as_uuid=True), nullable=False),
        sa.Column("result_data", postgresql.JSON(), nullable=False),
        sa.Column("format", sa.String(20), nullable=False),
        sa.Column("size", sa.Integer()),
        sa.Column("created_at", sa.DateTime(), nullable=False),
        sa.Column("updated_at", sa.DateTime(), nullable=False),
        sa.ForeignKeyConstraint(["query_id"], ["queries.id"])
    )
    
    # Create datasets table
    op.create_table(
        "datasets",
        sa.Column("id", postgresql.UUID(as_uuid=True), primary_key=True),
        sa.Column("owner_id", postgresql.UUID(as_uuid=True), nullable=False),
        sa.Column("name", sa.String(100), nullable=False),
        sa.Column("description", sa.Text()),
        sa.Column("schema", postgresql.JSON(), nullable=False),
        sa.Column("row_count", sa.Integer()),
        sa.Column("file_size", sa.Integer()),
        sa.Column("format", sa.String(20), nullable=False),
        sa.Column("metadata", postgresql.JSON()),
        sa.Column("is_public", sa.Boolean(), nullable=False, default=False),
        sa.Column("created_at", sa.DateTime(), nullable=False),
        sa.Column("updated_at", sa.DateTime(), nullable=False),
        sa.ForeignKeyConstraint(["owner_id"], ["users.id"])
    )
    
    # Create data_tables table
    op.create_table(
        "data_tables",
        sa.Column("id", postgresql.UUID(as_uuid=True), primary_key=True),
        sa.Column("dataset_id", postgresql.UUID(as_uuid=True), nullable=False),
        sa.Column("name", sa.String(100), nullable=False),
        sa.Column("description", sa.Text()),
        sa.Column("schema", postgresql.JSON(), nullable=False),
        sa.Column("row_count", sa.Integer()),
        sa.Column("primary_key", sa.String(100)),
        sa.Column("indexes", postgresql.JSON()),
        sa.Column("constraints", postgresql.JSON()),
        sa.Column("created_at", sa.DateTime(), nullable=False),
        sa.Column("updated_at", sa.DateTime(), nullable=False),
        sa.ForeignKeyConstraint(["dataset_id"], ["datasets.id"])
    )
    
    # Create api_keys table
    op.create_table(
        "api_keys",
        sa.Column("id", postgresql.UUID(as_uuid=True), primary_key=True),
        sa.Column("user_id", postgresql.UUID(as_uuid=True), nullable=False),
        sa.Column("key", sa.String(64), unique=True, nullable=False),
        sa.Column("name", sa.String(100), nullable=False),
        sa.Column("is_active", sa.Boolean(), nullable=False, default=True),
        sa.Column("expires_at", sa.DateTime()),
        sa.Column("last_used_at", sa.DateTime()),
        sa.Column("permissions", postgresql.JSON()),
        sa.Column("created_at", sa.DateTime(), nullable=False),
        sa.Column("updated_at", sa.DateTime(), nullable=False),
        sa.ForeignKeyConstraint(["user_id"], ["users.id"])
    )
    
    # Create audit_logs table
    op.create_table(
        "audit_logs",
        sa.Column("id", postgresql.UUID(as_uuid=True), primary_key=True),
        sa.Column("user_id", postgresql.UUID(as_uuid=True)),
        sa.Column("action", sa.String(50), nullable=False),
        sa.Column("resource_type", sa.String(50), nullable=False),
        sa.Column("resource_id", sa.String(50)),
        sa.Column("status", sa.String(20), nullable=False),
        sa.Column("details", postgresql.JSON()),
        sa.Column("ip_address", sa.String(45)),
        sa.Column("user_agent", sa.String(255)),
        sa.Column("created_at", sa.DateTime(), nullable=False),
        sa.Column("updated_at", sa.DateTime(), nullable=False),
        sa.ForeignKeyConstraint(["user_id"], ["users.id"])
    )
    
    # Create indexes
    op.create_index("ix_users_email", "users", ["email"])
    op.create_index("ix_users_username", "users", ["username"])
    op.create_index("ix_queries_user_id", "queries", ["user_id"])
    op.create_index("ix_query_results_query_id", "query_results", ["query_id"])
    op.create_index("ix_datasets_owner_id", "datasets", ["owner_id"])
    op.create_index("ix_data_tables_dataset_id", "data_tables", ["dataset_id"])
    op.create_index("ix_api_keys_user_id", "api_keys", ["user_id"])
    op.create_index("ix_api_keys_key", "api_keys", ["key"])
    op.create_index("ix_audit_logs_user_id", "audit_logs", ["user_id"])
    op.create_index("ix_audit_logs_resource_type", "audit_logs", ["resource_type"])

def downgrade():
    # Drop indexes
    op.drop_index("ix_audit_logs_resource_type")
    op.drop_index("ix_audit_logs_user_id")
    op.drop_index("ix_api_keys_key")
    op.drop_index("ix_api_keys_user_id")
    op.drop_index("ix_data_tables_dataset_id")
    op.drop_index("ix_datasets_owner_id")
    op.drop_index("ix_query_results_query_id")
    op.drop_index("ix_queries_user_id")
    op.drop_index("ix_users_username")
    op.drop_index("ix_users_email")
    
    # Drop tables
    op.drop_table("audit_logs")
    op.drop_table("api_keys")
    op.drop_table("data_tables")
    op.drop_table("datasets")
    op.drop_table("query_results")
    op.drop_table("queries")
    op.drop_table("users")
