"""
SQLAlchemy models for ADPA framework.
"""
from typing import Optional, Dict, Any
from datetime import datetime
import uuid
from sqlalchemy import Column, Integer, String, DateTime, Text, Boolean, ForeignKey, JSON
from sqlalchemy.orm import declarative_base, relationship
from sqlalchemy.dialects.postgresql import UUID

Base = declarative_base()

class TimestampMixin:
    """Mixin to add timestamp columns."""
    
    created_at = Column(
        DateTime,
        default=datetime.utcnow,
        nullable=False
    )
    updated_at = Column(
        DateTime,
        default=datetime.utcnow,
        onupdate=datetime.utcnow,
        nullable=False
    )

class User(Base, TimestampMixin):
    """User model."""
    
    __tablename__ = "users"
    
    id = Column(UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    username = Column(String(50), unique=True, nullable=False)
    email = Column(String(255), unique=True, nullable=False)
    password_hash = Column(String(255), nullable=False)
    is_active = Column(Boolean, default=True, nullable=False)
    last_login = Column(DateTime)
    preferences = Column(JSON)
    
    # Relationships
    queries = relationship("Query", back_populates="user")
    datasets = relationship("Dataset", back_populates="owner")

class Query(Base, TimestampMixin):
    """Query model."""
    
    __tablename__ = "queries"
    
    id = Column(UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    user_id = Column(UUID(as_uuid=True), ForeignKey("users.id"), nullable=False)
    natural_query = Column(Text, nullable=False)
    sql_query = Column(Text, nullable=False)
    execution_time = Column(Integer)  # in milliseconds
    row_count = Column(Integer)
    status = Column(String(20), nullable=False)
    error_message = Column(Text)
    metadata = Column(JSON)
    
    # Relationships
    user = relationship("User", back_populates="queries")
    results = relationship("QueryResult", back_populates="query")

class QueryResult(Base, TimestampMixin):
    """Query result model."""
    
    __tablename__ = "query_results"
    
    id = Column(UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    query_id = Column(UUID(as_uuid=True), ForeignKey("queries.id"), nullable=False)
    result_data = Column(JSON, nullable=False)
    format = Column(String(20), nullable=False)
    size = Column(Integer)  # in bytes
    
    # Relationships
    query = relationship("Query", back_populates="results")

class Dataset(Base, TimestampMixin):
    """Dataset model."""
    
    __tablename__ = "datasets"
    
    id = Column(UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    owner_id = Column(UUID(as_uuid=True), ForeignKey("users.id"), nullable=False)
    name = Column(String(100), nullable=False)
    description = Column(Text)
    schema = Column(JSON, nullable=False)
    row_count = Column(Integer)
    file_size = Column(Integer)  # in bytes
    format = Column(String(20), nullable=False)
    metadata = Column(JSON)
    is_public = Column(Boolean, default=False, nullable=False)
    
    # Relationships
    owner = relationship("User", back_populates="datasets")
    tables = relationship("DataTable", back_populates="dataset")

class DataTable(Base, TimestampMixin):
    """Data table model."""
    
    __tablename__ = "data_tables"
    
    id = Column(UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    dataset_id = Column(UUID(as_uuid=True), ForeignKey("datasets.id"), nullable=False)
    name = Column(String(100), nullable=False)
    description = Column(Text)
    schema = Column(JSON, nullable=False)
    row_count = Column(Integer)
    primary_key = Column(String(100))
    indexes = Column(JSON)
    constraints = Column(JSON)
    
    # Relationships
    dataset = relationship("Dataset", back_populates="tables")

class APIKey(Base, TimestampMixin):
    """API key model."""
    
    __tablename__ = "api_keys"
    
    id = Column(UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    user_id = Column(UUID(as_uuid=True), ForeignKey("users.id"), nullable=False)
    key = Column(String(64), unique=True, nullable=False)
    name = Column(String(100), nullable=False)
    is_active = Column(Boolean, default=True, nullable=False)
    expires_at = Column(DateTime)
    last_used_at = Column(DateTime)
    permissions = Column(JSON)

class AuditLog(Base, TimestampMixin):
    """Audit log model."""
    
    __tablename__ = "audit_logs"
    
    id = Column(UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    user_id = Column(UUID(as_uuid=True), ForeignKey("users.id"))
    action = Column(String(50), nullable=False)
    resource_type = Column(String(50), nullable=False)
    resource_id = Column(String(50))
    status = Column(String(20), nullable=False)
    details = Column(JSON)
    ip_address = Column(String(45))
    user_agent = Column(String(255))
